import java.util.*;  

public class Heapsort<T extends Comparable<? super T>> {
	
	   /**
	    *  Sorts a specified array of objects into ascending order, according to
	    *  the element class's implementation of the Comparable interface.
	    *  The worstTime(n) is O(n * log n).
	    *
	    *  @param x  the array to be sorted.  
	    *
	    */ 
	
	
	  public static <T extends Comparable<? super T>> void ListHeapSort (T[] x) {
		  // uso implementazione mediante ArrayList (*)
		   int dim = x.length;
		   
		   //costruzione MIN-HEAP
		   BinaryHeap<T> heap = new ListBinaryHeap<T>( x ); // (*)
		   for (int c = 0; c < dim ; c++)
			   x[c]= heap.deleteMin(); 
		   } // method ListHeapSort
	  
	  public static <T extends Comparable<? super T>> void ListHeapSort (T [] x, Comparator<T> comparator) {
		   int dim = x.length;  
		   //costruzione MIN-HEAP
		   BinaryHeap<T> heap = new ListBinaryHeap<T> ( x, comparator );
		   for (int c = 0; c < dim ; c++)
			   x[c]= heap.deleteMin(); 
		   } // method ListHeapSort
	  
	  public static <T extends Comparable<? super T>> void ArrayHeapSort (T[] x) {
		
	   int dim = x.length;
	   
	   //costruzione MIN-HEAP
	   BinaryHeap<T> heap = new ArrayBinaryHeap<T> ( x );
	   for (int c = 0; c < dim ; c++)
		   x[c]= heap.deleteMin(); 
	   } // method heapSort
	  
	  
	  
	    
	  /**
	    *  Sorts a specified array of objects into ascending order, according to
	    *  the element class's implementation of the Comparable interface.
	    *  The worstTime(n) is O(n * log n).
	    *
	    *  @param x  the array to be sorted.  
	    *
	    */ 
	  public static <T extends Comparable<? super T>> void heapSort (T [] x) {
		
	   int dim = x.length;

	   // heapify x - costruzione MAX-HEAP binario
	   for( int i = (dim - 2) / 2; i >= 0 ; i-- )
	         fixHeap( x,dim,i);

	   for (int c = dim-1; c > 0; c--) {

		T max = x[0]; 
		//deleteMax
	        x[0] = x[ --dim ];
	        fixHeap(x,c,0);
		x[c]=max;
		
	    }

	   } // method heapSort 	
	   
	   
	    private static <T extends Comparable<? super T>> void  fixHeap(T[] x, int currentSize, int hole )
	    {
	        int child;
	        T tmp = x[ hole ];

	        for( ; (hole * 2 + 1) <= currentSize -1; hole = child )
	        {
	            child = hole * 2 +1 ;
	            if( child != currentSize-1 &&
	                    (x[ child + 1 ]).compareTo( x[ child ] ) > 0 )
	                child++;
	            if( (x[ child ]).compareTo( tmp ) > 0 )
	                x[ hole ] = x[ child ];
	            else
	                break;
	        }
	        x[ hole ] = tmp;
	    }
	    
	    
	  
	  
	   public static void main(String[] args)
	    {
		   int L = 6000, MAXINT = 100000, NUMSIM = 5000;  //TEST
		   
		   Integer[] numbers = new Integer[L], numbersCopy = new Integer[L];

	      	double startTime, endTime;
	      	// tempi di esecuzione totale in ms
	      	double time_s=0, time_h=0, time_Ah=0, time_Lh=0;
	      	
	        Random r = new Random ();
	        for (int count=0; count < NUMSIM; count++) {

	  	// GENERAZIONE ARRAY RANDOM 
	       	for (int i = 0; i < L; i++) numbers[i]=r.nextInt(MAXINT);
	  	
	    
	  	//HEAPSORT
	  	//System.arraycopy(numbers, 0, numbersCopy, 0, L);
	    numbersCopy=numbers.clone();
	  	startTime = System.currentTimeMillis(); //The currentTimeMillis() method returns the current time in milliseconds.
	  		heapSort(numbersCopy);
	  	endTime = System.currentTimeMillis();
	  	time_h += endTime - startTime;  //per avere in sec /1000
	  	//test correttezza
	  	for(int i=0; i<L-1; i++) if (numbersCopy[i]>numbersCopy[i+1]) {
	  		System.out.println("heapSort errato!");
	  		return; }
	  	
	  	
	  	//ARRAY-HEAPSORT
	    numbersCopy=numbers.clone();
	  	startTime = System.currentTimeMillis(); //The currentTimeMillis() method returns the current time in milliseconds.
	  		ArrayHeapSort(numbersCopy);
	  	endTime = System.currentTimeMillis();
	  	time_Ah += endTime - startTime;  //per avere in sec /1000
	  	//test correttezza
	  	for(int i=0; i<L-1; i++) if (numbersCopy[i]>numbersCopy[i+1]) {
	  		System.out.println("heapSort errato!");
	  		return; }
	  	
	  	//LIST-HEAPSORT
	    numbersCopy=numbers.clone();
	  	startTime = System.currentTimeMillis(); //The currentTimeMillis() method returns the current time in milliseconds.
	  		ListHeapSort(numbersCopy);
	  	endTime = System.currentTimeMillis();
	  	time_Lh += endTime - startTime;  //per avere in sec /1000
	  	//test correttezza
	  	for(int i=0; i<L-1; i++) if (numbersCopy[i]>numbersCopy[i+1]) {
	  		System.out.println("heapSort errato!");
	  		return; }
	  	
	  
	  	
		//SORT
		System.arraycopy(numbers, 0, numbersCopy, 0, L);
		startTime = System.currentTimeMillis();
	        Arrays.sort(numbersCopy);
		endTime = System.currentTimeMillis();
		time_s += endTime - startTime;
		
	}
		
		System.out.println("\n\nTemp medio di esecuzione su " + NUMSIM + " array di " + L + " interi minori di " + MAXINT);
		System.out.println("Arrays.sort:\t\t" + time_s/NUMSIM + " msec");
		System.out.println("\nHeapSort:\t\t" + time_h/NUMSIM + " msec");
		System.out.println("\nArrayHeapSort:\t\t" + time_Ah/NUMSIM + " msec");
		System.out.println("\nListHeapSort:\t\t" + time_Lh/NUMSIM + " msec");


   } // method main

}




